@@ -1,9 +1,7 @@
 from __future__ import annotations
 
 import re
-from contextvars import ContextVar
-from functools import cached_property
-from typing import Any, Dict, List, Optional, Pattern, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Pattern, Union
 
 import stringcase
 import yaml
@@ -17,11 +15,34 @@ from datamodel_code_generator.parser.openapi import OpenAPIParser as OpenAPIMode
 from datamodel_code_generator.types import DataType
 from pydantic import BaseModel, root_validator
 
-MODEL_PATH = ".models"
+if TYPE_CHECKING:
+    from typing import Any, Callable
 
-model_path_var: ContextVar[str] = ContextVar('model_path', default=MODEL_PATH)
+    cached_property: Callable[..., Any]
+else:
+    try:
+        from functools import cached_property
+    except ImportError:
+        # For Python3.7
+        from cached_property import cached_property
 
-APPLICATION_JSON_PATTERN: Pattern[str] = re.compile(r'^application/.*json$')
+
+RE_APPLICATION_JSON_PATTERN: Pattern[str] = re.compile(r'^application/.*json$')
+
+
+def get_ref_body(
+    ref: str, openapi_model_parser: OpenAPIModelParser, components: Dict[str, Any]
+) -> Dict[str, Any]:
+    if ref.startswith('#/components'):
+        return get_model_by_path(components, ref[13:].split('/'))
+    elif ref.startswith('http://') or ref.startswith('https://'):
+        if '#/' in ref:
+            url, path = ref.rsplit('#/', 1)
+            ref_body = openapi_model_parser._get_ref_body(url)
+            return get_model_by_path(ref_body, path.split('/'))
+        else:
+            return openapi_model_parser._get_ref_body(ref)
+    raise NotImplementedError(f'ref={ref} is not supported')
 
 
 class CachedPropertyModel(BaseModel):
@@ -91,7 +112,7 @@ class Operation(CachedPropertyModel):
     imports: List[Import] = []
     security: Optional[List[Dict[str, List[str]]]] = None
     components: Dict[str, Any] = {}
-    open_api_model_parser: OpenAPIModelParser
+    openapi_model_parser: OpenAPIModelParser
 
     @cached_property
     def root_path(self) -> UsefulStr:
@@ -110,7 +131,7 @@ class Operation(CachedPropertyModel):
         for requests in self.request_objects:
             for content_type, schema in requests.contents.items():
                 # TODO: support other content-types
-                if re.match(APPLICATION_JSON_PATTERN, content_type):
+                if RE_APPLICATION_JSON_PATTERN.match(content_type):
                     data_type = self.get_data_type(schema)
                     arguments.append(
                         # TODO: support multiple body
@@ -130,15 +151,20 @@ class Operation(CachedPropertyModel):
     def request_objects(self) -> List[Request]:
         requests: List[Request] = []
         contents: Dict[str, JsonSchemaObject] = {}
-        for content_type, obj in self.requestBody.get('content', {}).items():
+        ref: Optional[str] = self.requestBody.get('$ref')
+        if ref:
+            request_body = get_ref_body(ref, self.openapi_model_parser, self.components)
+        else:
+            request_body = self.requestBody
+        for content_type, obj in request_body.get('content', {}).items():
             contents[content_type] = (
                 JsonSchemaObject.parse_obj(obj['schema']) if 'schema' in obj else None
             )
             requests.append(
                 Request(
-                    description=self.requestBody.get("description"),
+                    description=request_body.get("description"),
                     contents=contents,
-                    required=self.requestBody.get("required") is True,
+                    required=request_body.get("required") is True,
                 )
             )
         return requests
@@ -148,9 +174,9 @@ class Operation(CachedPropertyModel):
         responses: List[Response] = []
         for status_code, detail in self.responses.items():
             ref: Optional[str] = detail.get('$ref')
-            if ref and ref.startswith('#/components/'):
-                content: Dict[str, Any] = get_model_by_path(
-                    self.components, ref[13:].split('/')
+            if ref:
+                content = get_ref_body(
+                    ref, self.openapi_model_parser, self.components
                 ).get("content", {})
             else:
                 content = detail.get("content", {})
@@ -210,11 +236,11 @@ class Operation(CachedPropertyModel):
 
     def get_data_type(self, schema: JsonSchemaObject) -> DataType:
         if schema.ref:
-            data_type = self.open_api_model_parser.get_ref_data_type(schema.ref)
+            data_type = self.openapi_model_parser.get_ref_data_type(schema.ref)
             data_type.imports_.append(
                 Import(
                     # TODO: Improve import statements
-                    from_=model_path_var.get(),
+                    from_=self.openapi_model_parser.current_source_path,  # model_path_var.get(),
                     import_=data_type.type,
                 )
             )
@@ -222,19 +248,17 @@ class Operation(CachedPropertyModel):
         elif schema.is_array:
             # TODO: Improve handling array
             items = schema.items if isinstance(schema.items, list) else [schema.items]
-            return self.open_api_model_parser.data_type(
+            return self.openapi_model_parser.data_type(
                 data_types=[self.get_data_type(i) for i in items], is_list=True
             )
-        return self.open_api_model_parser.get_data_type(schema)
+        return self.openapi_model_parser.get_data_type(schema)
 
     def get_parameter_type(
         self, parameter: Dict[str, Union[str, Dict[str, str]]], snake_case: bool
     ) -> Argument:
         ref: Optional[str] = parameter.get('$ref')  # type: ignore
         if ref:
-            if not ref.startswith('#/components'):
-                raise NotImplementedError(f'{ref=} is not supported for parameters')
-            parameter = get_model_by_path(self.components, ref[13:].split('/'))
+            parameter = get_ref_body(ref, self.openapi_model_parser, self.components)
         name: str = parameter["name"]  # type: ignore
         orig_name = name
         if snake_case:
@@ -269,7 +293,7 @@ class Operation(CachedPropertyModel):
             # expect 2xx
             if response.status_code.startswith("2"):
                 for content_type, schema in response.contents.items():
-                    if re.match(APPLICATION_JSON_PATTERN, content_type):
+                    if RE_APPLICATION_JSON_PATTERN.match(content_type):
                         data_type = self.get_data_type(schema)
                         data_types.append(data_type)
                         self.imports.extend(data_type.imports_)
@@ -277,7 +301,7 @@ class Operation(CachedPropertyModel):
         if not data_types:
             return "None"
         if len(data_types) > 1:
-            return self.open_api_model_parser.data_type(data_types=data_types).type_hint
+            return self.openapi_model_parser.data_type(data_types=data_types).type_hint
         return data_types[0].type_hint
 
 
@@ -309,37 +333,39 @@ class Operations(BaseModel):
     path: UsefulStr
     security: Optional[List[Dict[str, List[str]]]] = []
     components: Dict[str, Any] = {}
-    open_api_model_parser: OpenAPIModelParser
+    openapi_model_parser: OpenAPIModelParser
 
     @root_validator(pre=True)
     def inject_path_and_type_to_operation(cls, values: Dict[str, Any]) -> Any:
         path: Any = values.get('path')
-        open_api_model_parser: OpenAPIModelParser = values.get('open_api_model_parser')
+        openapi_model_parser: OpenAPIModelParser = values.get('openapi_model_parser')
         return dict(
             **{
                 o: dict(
-                    **v,
+                    **values[o],
                     path=path,
                     type=o,
                     components=values.get('components', {}),
-                    open_api_model_parser=open_api_model_parser,
+                    openapi_model_parser=openapi_model_parser,
                 )
                 for o in OPERATION_NAMES
-                if (v := values.get(o))
+                if o in values
             },
             path=path,
             parameters=values.get('parameters', []),
             security=values.get('security'),
             components=values.get('components', {}),
-            open_api_model_parser=open_api_model_parser,
+            openapi_model_parser=openapi_model_parser,
         )
 
     @root_validator
     def inject_parameters_and_security_to_operation(cls, values: Dict[str, Any]) -> Any:
         security = values.get('security')
         for operation_name in OPERATION_NAMES:
-            if operation := values.get(operation_name):
-                if parameters := values.get('parameters'):
+            operation = values.get(operation_name)
+            if operation:
+                parameters = values.get('parameters')
+                if parameters:
                     operation.parameters.extend(parameters)
                 if security is not None and operation.security is None:
                     operation.security = security
@@ -352,17 +378,19 @@ class Path(CachedPropertyModel):
     operations: Optional[Operations] = None
     security: Optional[List[Dict[str, List[str]]]] = []
     components: Dict[str, Any] = {}
-    open_api_model_parser: OpenAPIModelParser
+    openapi_model_parser: OpenAPIModelParser
 
     @root_validator(pre=True)
     def validate_root(cls, values: Dict[str, Any]) -> Any:
-        if path := values.get('path'):
+        path = values.get('path')
+        if path:
             if isinstance(path, str):
-                if operations := values.get('operations'):
+                operations = values.get('operations')
+                if operations:
                     if isinstance(operations, dict):
                         security = values.get('security', [])
                         components = values.get('components', {})
-                        open_api_model_parser = values.get('open_api_model_parser')
+                        openapi_model_parser = values.get('openapi_model_parser')
                         return {
                             'path': path,
                             'operations': dict(
@@ -370,11 +398,11 @@ class Path(CachedPropertyModel):
                                 path=path,
                                 security=security,
                                 components=components,
-                                open_api_model_parser=open_api_model_parser,
+                                openapi_model_parser=openapi_model_parser,
                             ),
                             'security': security,
                             'components': components,
-                            'open_api_model_parser': open_api_model_parser,
+                            'openapi_model_parser': openapi_model_parser,
                         }
         return values
 
@@ -382,9 +410,9 @@ class Path(CachedPropertyModel):
     def exists_operations(self) -> List[Operation]:
         if self.operations:
             return [
-                operation
+                getattr(self.operations, operation_name)
                 for operation_name in OPERATION_NAMES
-                if (operation := getattr(self.operations, operation_name))
+                if getattr(self.operations, operation_name)
             ]
         return []
 
@@ -419,9 +447,9 @@ class OpenAPIParser:
     ) -> None:
         self.input_name: str = input_name
         self.input_text: str = input_text
+        self.openapi_model_parser: OpenAPIModelParser = OpenAPIModelParser(source='')
         if model_path:
-            model_path_var.set(model_path)
-        self.open_api_model_parser: OpenAPIModelParser = OpenAPIModelParser(source='')
+            self.openapi_model_parser.current_source_path = '.models'
 
     def parse(self) -> ParsedObject:
         openapi = yaml.safe_load(self.input_text)
@@ -449,7 +477,7 @@ class OpenAPIParser:
                     operations=operations,
                     security=security,
                     components=openapi.get('components', {}),
-                    open_api_model_parser=self.open_api_model_parser,
+                    openapi_model_parser=self.openapi_model_parser,
                 ).exists_operations
             ],
             info,
